<?php


namespace tpScriptVueCurd\base\controller;


use think\App;
use think\facade\Event;
use think\helper\Str;
use think\model\Collection;
use think\Request;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\enum\ActionType;
use tpScriptVueCurd\field\FilesField;
use tpScriptVueCurd\FieldCollection;
use tpScriptVueCurd\option\step\StepBase;
use tpScriptVueCurd\option\step\StepCollection;
use tpScriptVueCurd\traits\controller\BaseDel;
use tpScriptVueCurd\traits\controller\BaseEdit;
use tpScriptVueCurd\traits\controller\BaseIndex;
use tpScriptVueCurd\traits\controller\BaseShow;
use tpScriptVueCurd\traits\controller\Bpmn;
use tpScriptVueCurd\traits\controller\Childs;
use tpScriptVueCurd\traits\controller\Excel;
use tpScriptVueCurd\traits\controller\Export;
use tpScriptVueCurd\traits\controller\HaveChilds;
use tpScriptVueCurd\traits\controller\TreeIndex;
use tpScriptVueCurd\traits\controller\Vue;
use tpScriptVueCurd\traits\CurdParams;

/**
 * Trait Controller
 * @author tj 1079798840@qq.com
 * @property APP $app
 * @property Request $request
 * @property FieldCollection $fields;
 */
trait Controller
{
    use Vue,CurdParams,BaseIndex,TreeIndex,BaseShow,BaseEdit,BaseDel,HaveChilds,Childs,Excel,Export,Bpmn{
        Vue::initialize as vueInitialize;
    }

    /**
     * @var BaseModel
     */
    public $md;
    public ?FieldCollection $fields=null;
    /**
     * @var StepCollection|null|StepBase[]
     */
    public ?StepCollection $steps=null;
    public string $title='';


    public bool $dontShowTpl=false;
    public bool $dontShowTplAndGetData=false;
    public string $fetchPath;
    public string $tplPath='';


    public function initialize()
    {
        if(empty($this->app)){
            $this->app=app();
        }
        if(empty($this->request)){
            $this->request=$this->app->request;
        }
        if(empty($this->guid)){
            $this->guid=create_guid();
        }
        $this->vueInitialize();


     
        $this->tplPath=getVCurdDir().'tpl'.DIRECTORY_SEPARATOR;
        $this->assign('jsPath','/tpscriptvuecurd/default.js');


        $this->init();
        $this->setInfo(clone $this->md);
        $this->steps=$this->md->getStepConfig();
        $this->fields=$this->steps?null:$this->md->getFieldConfig();

    }


    /**
     * 控制器初始化配置
     */
    abstract public function init():void;



    /**
     * 生成控制器对象
     * @param BaseModel|null $model
     * @return static
     */
    public static function make(BaseModel $model=null):self{
        $return=new static(app());
        is_null($model)||$return->md=$model;
        return $return;
    }





    /**
     * 设置要编辑或显示的数据对象
     * @return void
     * @throws \think\Exception
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    protected function setInfoById(int $id=null): void
    {
        $getParamId=function ($key):?int{
            if($this->request->isPost()){
                //先以post为准
                return $this->request->post($key.'/d')?:$this->request->get($key.'/d')?:$this->request->param($key.'/d');
            }
            //先以get为准
            return $this->request->get($key.'/d')?:$this->request->param($key.'/d');
        };

        $id=$id?:$getParamId('id');

        $parentCont=$this->getParentController();
        $parentInfo=$this->getInfo()->getParentInfo();
        if($id){
            $info=$this->md->find($id);
            $info||$this->errorOut('未找到相关数据信息');
            if($this->steps){
                $this->md->where('id',$id)->where($this->steps->doWhere(ActionType::Edit))->value('id')||$this->errorOut('您不能操作此数据');
            }
            if($parentCont){
                $parentInfo=$parentCont->md->find($info[$this->md::parentField()]);
            }
        }else{
            $info=clone $this->md;
            if($this->treePidField&&$pid=$getParamId('base_id')){
                $info[$this->treePidField]=$pid;
            }
            if($parentCont) {
                $baseId=$getParamId('base_id');
                $baseId || $this->errorOut('缺少必要参数');
                $info[$this->md::parentField()] = $baseId;
                $parentInfo = $parentCont->md->find($baseId);
            }
        }

        if($parentCont&&empty($parentInfo)){
            $this->errorOut('未找到所属父表数据');
        }

        $info->setParentInfo($parentInfo);
        $this->setInfo($info);
    }


    /**
     * 显示模板内容
     * @param string $file 直接在控制器下面的模板位置添加模板文件就可替换默认的模板，或者使用fetchPath
     * @param array $data
     * @return array|mixed
     */
    protected function showTpl(string $file,array $data): mixed
    {
        FilesField::setShowFileInfos();
        if($this->dontShowTplAndGetData){
            return $data;
        }
        if($this->dontShowTpl){
            $this->successOut($data);
        }

        if(isset($this->fetchPath)&&$this->fetchPath!==''){
            return $this->fetch($this->fetchPath,$data);
        }

        $appName = $this->app->http->getName();
        $view    = $this->app->view->getConfig('view_dir_name');
        $depr =$this->app->view->getConfig('view_depr');

        $path = $this->app->getAppPath() . $view . DIRECTORY_SEPARATOR;
        if (!is_dir($this->app->getAppPath() . $view)&&$appName) {
            $path .= $appName . DIRECTORY_SEPARATOR;
        }
        $controller = $this->app->request->controller();
        if (strpos($controller, '.')) {
            $pos        = strrpos($controller, '.');
            $controller = substr($controller, 0, $pos) . '.' . Str::snake(substr($controller, $pos + 1));
//            $controller_name=Str::snake(substr($controller, $pos + 1));
        } else {
            $controller = Str::snake($controller);
//            $controller_name=$controller;
        }
        $template=$file?:Str::snake( $this->app->request->action());
        $path .= str_replace('.', DIRECTORY_SEPARATOR, $controller) . $depr . ($file ?: Str::snake($this->app->request->action())) . '.vue';
        if(file_exists($path)){
            return $this->fetch(str_replace('.', '/', $controller).'/'.$template,$data);
        }
        $tplPath=static::getTplPath();
        return $this->fetch($tplPath.$file.'.vue',$data);
    }



    protected function onSetInfo(BaseModel $info): void
    {
        Event::trigger('onSetInfoByController',[$this,$info]);

        if(empty($this->childControllers)){
            return;
        }
        foreach ($this->childControllers as $v){
            $v->getInfo()?->setParentInfo($this->getInfo());
        }
    }


    /**
     * @param Collection $infoList
     * @return void
     */
    public function onSetInfoList(Collection $infoList): void
    {
        Event::trigger('onSetInfoListByController',[$this,$infoList]);
        $this->md->setInfoList($infoList);
    }

}