<?php


namespace tpScriptVueCurd\field;
use think\Exception;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\ExcelFieldTpl;
use tpScriptVueCurd\filter\EmptyFilter;
use tpScriptVueCurd\ModelField;
use tpScriptVueCurd\option\generate_table\GenerateColumnOption;
use tpScriptVueCurd\tool\field_tpl\FieldTpl;


/**
 * 密码输入
 * Class PasswordField
 * @author tj 1079798840@qq.com
 * @package tpScriptVueCurd
 */
class PasswordField extends ModelField
{
    protected string $defaultFilterClass=EmptyFilter::class;
    protected bool $showInList=false;

    protected bool $canExport=false;//不能导出此字段数据


    /**
     * 是否在后台列表中显示出来
     * @param bool|null $showInList
     * @return $this|bool
     */
    public function showInList(bool $showInList=null): bool|static
    {
        if($showInList===true){
            throw new \think\Exception('字段类型['.$this->getType().']不能设置showInList未true');
        }
        return $this->doAttr('showInList',$showInList);
    }

    /**
     * 设置保存的值
     * @param array $data 数据值集合
     * @param BaseModel $old
     * @return $this
     */
    public function setSaveVal(array $data,BaseModel $old): self
    {
        if(isset($data[$this->name()])){
            $this->save=trim($data[$this->name()]);
            $this->defaultCheckRequired($this->save);
        }else{
            $this->defaultCheckRequired($this->nullVal());
        }
        return $this;
    }

    /**
     * 模板导入备注
     * @param ExcelFieldTpl $excelFieldTpl
     * @return void
     */
    public function excelTplExplain(ExcelFieldTpl $excelFieldTpl):void{
        $excelFieldTpl->width=22;
    }

    /**
     * 显示时要处理的数据
     * @param array $dataBaseData
     */
    public function doShowData(array &$dataBaseData): void
    {
        $name=$this->name();
        if(isset($dataBaseData[$name])){
            $dataBaseData[$name]='••••••';
        }
    }


    public static function componentUrl(): FieldTpl
    {
        return new FieldTpl(static::class,
            edit:'/tpscriptvuecurd/field/password/edit.js'
        );
    }

    /**
     * 数据库字段生成配置
     * @param  GenerateColumnOption $option
     * @return void
     */
    public function getGenerateColumnConfig(GenerateColumnOption $option):void{
        $option->setTypeVarchar(128);
    }
}