<?php

use think\facade\Env;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\option\ConfirmException;
use tpScriptVueCurd\option\step\StepBase;


/**
 * static 防止重复访问数据库 获取登录人员信息
 * @return array []
 */
function staticTpScriptVueCurdGetLoginData():array{
    static $data;
    if(!$data){
        $data=tpScriptVueCurdGetLoginData();
    }
    return $data;
}

/**
 * 递归合并两个数组
 * @param array $old
 * @param array $new
 * @return array
 */
function vueCurdMergeArrays(array $old,array $new):array{
    foreach ($old as $k=>$v){
        if(isset($new[$k])){
            if(is_array($v)&&is_array($new[$k])){
                $v=vueCurdMergeArrays($v,$new[$k]);
            }else{
                $v=$new[$k];
            }
            $old[$k]=$v;
        }
    }
    foreach ($new as $k=>$v){
        if(!isset($old[$k])){
            $old[$k]=$v;
        }
    }
    return $old;
}


/**
 * 获取最后一步的详细信息
 * @param array|string|BaseModel $stepOrInfo
 * @return array|null
 */
function endStep($stepOrInfo): ?array
{
    if(empty($stepOrInfo)){
        return null;
    }

    if(is_array($stepOrInfo)){
        return end($stepOrInfo);
    }

    if(is_string($stepOrInfo)){
        return endStep(json_decode($stepOrInfo,true));
    }

    if($stepOrInfo instanceof \tpScriptVueCurd\base\model\BaseModel){
        if(!$stepOrInfo::hasStepField()){
            return null;
        }

        return endStep($stepOrInfo[$stepOrInfo::getStepField()]);
    }

    return null;
}


/**
 * 获取最后一步的步骤的值
 * @param $stepOrInfo
 * @return string|null
 */
function endStepVal($stepOrInfo):?string{
    if(empty($stepOrInfo)&&$stepOrInfo!==0&&$stepOrInfo!=='0'){
        return null;
    }

    $endStepInfoVal=function($val){
        $endStep=endStep($val);
        if(is_null($endStep)){
            return null;
        }
        return $endStep['step'];
    };

    if(is_string($stepOrInfo)){
        $json=json_decode($stepOrInfo,true);
        if($json===null){
            return $stepOrInfo;
        }
        return $endStepInfoVal($json);
    }

    if(is_array($stepOrInfo)){
        return $endStepInfoVal($stepOrInfo);
    }

    if($stepOrInfo instanceof \tpScriptVueCurd\base\model\BaseModel){
        if($stepOrInfo::hasCurrentStepField()&&$stepOrInfo[$stepOrInfo::getCurrentStepField()]!=='') {
            return $stepOrInfo[$stepOrInfo::getCurrentStepField()];
        }
        if($stepOrInfo::hasStepPastsField()&&$stepOrInfo[$stepOrInfo::getStepPastsField()]!=='') {
            $steps=explode(',',$stepOrInfo[$stepOrInfo::getStepPastsField()]);
            return end($steps);
        }
        if($stepOrInfo::hasStepField()){
            return $endStepInfoVal($stepOrInfo[$stepOrInfo::getStepField()]);
        }
    }

    return null;
}

/**
 * 判断对象数据库存的当前步骤是否是 $step
 * @param StepBase|string $step
 * @param array|string|BaseModel $stepOrInfo
 * @return bool
 */
function eqEndStep(StepBase|string $step, array|BaseModel|string $stepOrInfo): bool
{
    if($step instanceof StepBase){
        $step=$step::name();
    }
    $endStep=endStepVal($stepOrInfo);
    if($endStep===null){
        return false;
    }
    return $step===$endStep;
}

/**
 * 获取数据所有已完成的步骤，已退回的步骤不算在里面
 * @param array|string|BaseModel $stepOrInfo
 * @return array
 */
function getStepPasts(array|BaseModel|string $stepOrInfo):array{
    if(is_string($stepOrInfo)){
        $stepHistory=json_decode($stepOrInfo,true);
    }else if($stepOrInfo instanceof BaseModel){
        $stepHistory=json_decode($stepOrInfo[$stepOrInfo::getStepField()],true);
    }else if(is_array($stepOrInfo)){
        $stepHistory=$stepOrInfo;
    }else{
        return [];
    }

    $stepValues=[];
    if($stepHistory)foreach ($stepHistory as $v){
        if(empty($v['back'])){
            $stepValues[]=(string)$v['step'];
        }else{
            $stepValues=array_slice($stepValues,0,count($stepValues)-$v['back']);
        }
    }
    return $stepValues;
}

/**
 * 判断对象数据库存的步骤是否已经过了 $step，会自动去掉退回的一些步骤
 * @param StepBase|string $step
 * @param array|string|BaseModel $stepOrInfo
 * @return bool
 */
function stepCheckIsPast(StepBase|string $step, array|BaseModel|string $stepOrInfo):bool{
    if($step instanceof StepBase){
        $step=$step::name();
    }
    return in_array((string)$step,getStepPasts($stepOrInfo),true);
}

if (!function_exists('create_guid')) {
    function create_guid(): string
    {
        // 1、去掉中间的“-”，长度有36变为32
        // 2、字母由“大写”改为“小写”
        if (function_exists('com_create_guid') === true) {
            return strtolower(str_replace('-', '', trim(com_create_guid(), '{}')));
        }
        return sprintf('%04x%04x%04x%04x%04x%04x%04x%04x', mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(16384, 20479), mt_rand(32768, 49151), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535));
    }
}

/**
 * 框架根目录地址
 * @return string
 */
function getVCurdDir():string{
    return dirname(__DIR__).DIRECTORY_SEPARATOR;
}

/**
 * 是否开启了debug
 * @return bool
 */
function appIsDebug():bool{
    return \think\facade\App::isDebug();
}

/**
 * 报错时，是否直接抛出异常
 * @throws Exception
 */
function errorShowThrow(\Exception $error=null):void{
    if(is_null($error)){
        return;
    }
    if($error instanceof ConfirmException||in_array((int)$error->getCode(),\tpScriptVueCurd\enum\ErrorCode::getDebugNotOutErrInfos(),true)){
        return;
    }
    $classPath=get_class($error);
    if(in_array($classPath,['think\Exception','Exception'])){
        return;
    }
    throw $error;
}

/**
 *  request 不能设置为 null 的问题解决
 * @param $val
 * @return mixed
 */
function requestFilterNull($val): mixed
{
    return $val;
}

function vueCurdVersion():string{
    $def='1.0.1';

    if(!is_file(root_path().'composer.lock')){
        return $def;
    }
    $content=file_get_contents(root_path().'composer.lock');
    $content = json_decode($content, true, 512, JSON_THROW_ON_ERROR);
    if(!$content){
        return $def;
    }

    if(isset($content['packages'])){
        foreach ($content['packages'] as $v){
            if($v['name']==='jc/tp-script-vue-curd'){
                return $v['version'];
            }
        }
    }

    if(isset($content['packages-dev'])){
        foreach ($content['packages-dev'] as $v){
            if($v['name']==='jc/tp-script-vue-curd'){
                return $v['version'];
            }
        }
    }
    return $def;
}