<?php

namespace tpScriptVueCurd\option\step;

use think\db\Query;
use think\facade\Event;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\FieldCollection;

abstract class StepBase extends FieldCollection
{
    /**
     * 配置信息
     * @var StepConfig
     */
    public StepConfig $step;
    /**
     * 列表步骤中显示的标签
     * @var StepTag[]
     */
    protected array $tags=[];
    protected string $remark='';//将在列表步骤中显示的备注
    protected string $remarkBtnText='查看';
    protected string $listDirectSubmit='';//列表中直接提交，不经过编辑页面，按钮提示的字段。如果有，列表中直接提交


    private ?string $beforeStepClassByInfo=null;

    public function __construct(array $item = [])
    {
        $fields=$this->fields();
        if(is_string($fields)){
            $this->listDirectSubmit=$fields;
            $fields=[];
        }
        parent::__construct($item?:$fields);

        $this->step=new StepConfig();
        $this->config($this->step);
    }

    public static function name():string{
        return class_basename(static::class);
    }

    public function getTitle():string{
        return $this->step->title;
    }

    //当前信息上一步的Class
    public function getBeforeStepClassByInfo():?string{
        return $this->beforeStepClassByInfo;
    }

    abstract protected function fields(): array|string;


    /**
     * 定义config
     * @param StepConfig $config
     */
    abstract public function config(StepConfig $config): void;

    /**
     * @return array|callable[]|StepBeforeCheck[]
     */
    abstract protected function beforeSteps(): array;

    /**
     * 权限查询条件（满足条件时，才能显示此条数据信息，默认都能查看，多个步骤时条件是 or ）
     * @param Query $query
     */
    abstract public function authWhere(Query $query): void;


    protected function auth():bool{
        return true;
    }

    public function canEdit():bool{
        return false;
    }

    final public function checkNextAuth(): bool
    {
        return $this->checkAuth();
    }

    final public function checkEditAuth(): bool
    {
        return $this->checkAuth()&&$this->canEdit();
    }

    private function checkAuth(): bool
    {
        return $this->auth();
    }

    /**
     * 验证数据是否满足
     * @return bool
     * @throws \think\Exception
     */
    final public function checkBefores():bool{
        if(!$this->premiseByBeforeCheck()){
            return false;
        }
        $info=$this->getInfo();
        $currStep = $info === null ||empty($info->id) ? '' : endStepVal($info);


        $checkFunc= static function ($k, $v)use($currStep,$info):?bool{
            if (is_numeric($k)) {
                if (is_string($v)) {
                    if (is_subclass_of($v, self::class)) {
                        $name = $v::name();
                    } else {
                        throw new \think\Exception(static::class . '步骤配置不合法' . $k . '=>' . $v);
                    }
                    $v = true;
                } else if (is_bool($v)||is_callable($v)) {
                    $name = '';
                } else {
                    //不合法，跳过
                    return null;
                }
            } else if ($k === '') {
                $name = $k;
            } else if (is_subclass_of($k, self::class)) {
                $name = $k::name();
            } else {
                throw new \think\Exception(static::class . '步骤配置不合法' . $k . '=>' . $v);
            }
            if ($name !== $currStep) {
                return null;
            }
            if(is_array($v)){
                foreach ($v as $key=>$val){
                    if(!isset($info[$key])||(!is_string($info[$key])&&!is_numeric($info[$key]))||(string)$info[$key]!==(string)$val){
                        return false;
                    }
                }
                return !empty($v);
            }
            if (is_callable($v)) {
                return $v();
            }
            if (is_bool($v)) {
                return $v;
            }
            if($v instanceof StepBeforeCheck){
                return ($v->func)();
            }
            throw new \think\Exception(static::class . '步骤配置不合法' . $k . '=>未知类型' );
        };

        foreach ($this->beforeSteps() as $k => $v) {
            $res=$checkFunc($k,$v);
            if(is_null($res)){
                continue;
            }
            if($res){
                $this->beforeStepClassByInfo=is_numeric($k)?(is_bool($v)?'':$v):$k;
            }
            return $res;
        }
        return false;
    }

    protected function premiseByBeforeCheck():bool{
        //验证前执行
        return true;
    }

    protected function onSetInfo(?BaseModel $info): void
    {
        parent::onSetInfo($info);
        $this->config($this->step);
        Event::listen('onSetParentInfo',function ($info){
            $this->config($this->step);
        });
    }

    public function fieldsToArray():array{
        return parent::toArray();
    }
    public function column(?string $columnKey, string $indexKey = null): array
    {
        return array_column($this->fieldsToArray(), $columnKey, $indexKey);
    }


    public function getStepSaveJson():string{
        if(!$this->getInfo()){
            throw new \think\Exception('无数据信息调用getStepSaveJson');
        }
        $stepData=[
            'step'=>static::name(),
            'time'=>time(),
            'user'=>staticTpScriptVueCurdGetLoginData()['id'],
            'back'=>'0',//后退步数
        ];
        $stepsJson=$this->getInfo()[$this->getInfo()::getStepField()]??null;
        if($stepsJson){
            if(is_string($stepsJson)){
                $stepsArr= json_decode($stepsJson, true, 512, JSON_THROW_ON_ERROR);
            }else if(!is_array($stepsJson)){
                throw new \think\Exception('步骤参数错误');
            }else{
                $stepsArr=$stepsJson;
            }
            $nowStep=end($stepsArr);
            if($nowStep['step']!==static::name()){
                //新的步骤
                $stepsArr[]=$stepData;
            }
        }else{
            $stepsArr=[$stepData];
        }
        return json_encode($stepsArr, JSON_THROW_ON_ERROR);
    }


    /**
     * 修正 步骤的值
     * @param string|array $stepsJson
     * @return string
     * @throws \JsonException
     */
    public static function correctSteps(array|string$stepsJson):string{
        if(is_string($stepsJson)){
            $stepsArr= json_decode($stepsJson, true, 512, JSON_THROW_ON_ERROR);
        }else{
            $stepsArr=$stepsJson;
        }

        $keyArrs=[];
        foreach ($stepsArr as $k=>$v){
            if(!in_array($v['step'], $keyArrs, true)){
                $keyArrs[]=$v['step'];
                $v['back']=0;
            }else{
                $findIndex= array_search($v['step'], $keyArrs, true);
                $v['back']=count($keyArrs)-$findIndex-1;
                if($v['back']<0){
                    continue;
                }

                if($v['back']===0){
                    //已存在且步骤紧挨，代表步骤并没有改变
                    unset($stepsArr[$k]);
                    continue;
                }
                $keyArrs=array_slice($keyArrs,0,$findIndex+1);
            }
            $stepsArr[$k]=$v;
        }

        return json_encode(array_values($stepsArr), JSON_THROW_ON_ERROR);
    }


    /**
     * 获取所有的BeforeSteps逻辑,StepBeforeCheck格式
     * @return StepBeforeCheck[]
     * @throws \think\Exception
     */
    public function getBeforeSteps(): array
    {
        $funcs = $this->beforeSteps();

        foreach ($funcs as $key => $value) {
            if ($value instanceof StepBeforeCheck) {
                continue;
            }
            $oldKey=$key;
            if (is_numeric($key)) {
                if (is_string($value)) {
                    if (!is_subclass_of($value, self::class)) {
                        throw  new  \think\Exception(static::class . '步骤配置不合法' . $key . '=>' . $value);
                    }
                    $key = $value;
                    $value = true;
                } else if (!is_bool($value)&&!is_callable($value)) {
                    throw  new  \think\Exception(static::class . '步骤配置不合法' . $key . '=>' . $value);
                }
            } else if ($key !== '' && !is_subclass_of($key, self::class)) {
                throw  new  \think\Exception(static::class . '步骤配置不合法' . $key . '=>' . $value);
            }
            if (is_bool($value)) {
                $value = static fn()=>$value;
            }else if(is_array($value)){
                $value= static function ($info)use($value){
                    foreach ($value as $key=>$val){
                        if(!isset($info[$key])||(!is_string($info[$key])&&!is_numeric($info[$key]))||(string)$info[$key]!==(string)$val){
                            return false;
                        }
                    }
                    return !empty($value);
                };
            } else if (!is_callable($value)) {
                throw new \think\Exception(static::class . '步骤配置不合法' . $key . '=>' . $value);
            }
            if($oldKey!==$key){
                unset( $funcs[$oldKey]);
            }
            $funcs[$key] = StepBeforeCheck::make('', $value);
        }
        return $funcs;
    }

    public function toArray():array
    {
        $tags = [];
        foreach ($this->tags as $v) {
            $tags[] = $v->toArray();
        }

        return [
            'step' => static::name(),
            'title' => $this->getTitle(),
            'config' => $this->step->toArray(),
            'tags' => $tags,
            'remark' => $this->remark,
            'remarkBtnText' => $this->remarkBtnText,
            'listDirectSubmit' => $this->listDirectSubmit,
        ];
    }
}