<?php

namespace tpScriptVueCurd\option\step;

use think\Collection;
use think\db\Query;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\enum\ActionType;
use tpScriptVueCurd\traits\CurdParams;

/**
 * @property StepBase[] $items
 */
class StepCollection extends Collection
{
    use CurdParams;

    private array $config=[
        'showInList'=>true,
        'listFixed'=>'',//列表中，列是否浮动，'left'/'right'
        'width'=>0,//0为自动
        'showFilter'=>false,//是否启用步骤筛选筛选
        'currentFilterDefaultShow'=>true,//默认显示筛选
        'nextFilterDefaultShow'=>true,//显示筛选
    ];

    /**
     * @var callable|null $queryWhere
     */
    private $queryWhere=null;
    /**
     * @var callable[] $queryWhereOr
     */
    private $queryWhereOr=[];

    /**
     * 这几个操作要判断where权限
     * @var ActionType[]
     */
    private  $whereCheckActions=[ActionType::Edit,ActionType::Show,ActionType::Index];




    public function setConfig(string $key,$val):void{
        $this->config[$key]=$val;
    }
    public function getConfig(string $key=null){
        if(is_null($key)){
            return $this->config;
        }
        return $this->config[$key]??null;
    }


    public function getNextStep():?StepBase{
        $next=null;
        foreach ($this->items as $v){
            $beforeStep=$v->checkBefores();
            if($beforeStep){
                if($next){
                    throw new \think\Exception('[配置错误]同时满足步骤 '.$next->getTitle().' 与 '.$v->getTitle());
                }else{
                    $next=$v;
                }
            }
        }
        return $next;
    }


    public function getCurrentStep():?StepBase{
        $info=$this->getInfo();
        $currentStep=  endStepVal($info);
        if(is_null($currentStep)){
            return null;
        }
        foreach ($this->items as $v){
            if($v::name()===$currentStep){
                return $v;
            }
        }
        return null;
    }




    protected function onSetInfo(BaseModel $info): void
    {
        foreach ($this->items as $v){
            $v->setInfo($info);
        }
    }

    public function onSetInfoList(?\think\model\Collection $infoList): void
    {
        foreach ($this->items as $v){
            $v->setInfoList($infoList);
        }
    }


    /**
     * 步骤 or 条件，当角色不满足步骤时，却又要显示/查看相关数据，可在此处加入条件
     * @param callable|array|string $where
     * @return $this
     */
    public function setAuthWhereOr( $where,ActionType $actionType):self{
        $this->queryWhereOr[$actionType->value()]=$where;
        return $this;
    }

    /**
     * 设置数据满足条件
     * @param callable|array|string $where
     * @return $this
     */
    public function setWhere( $where):self{
        $this->queryWhere=$where;
        return $this;
    }
    /**
     * 获取数据查询（满足）条件
     * @param callable $where
     * @return $this
     */
    public function getWhere():array|callable{
        return   $this->queryWhere?$this->queryWhere:[];
    }

    /**
     * 获取/执行 数据查询（满足）条件，包含下面所有步骤的条件
     * @return null|callable
     */
    public function doWhere(?ActionType $actionType):?callable{
                if($actionType&&!in_array($actionType,$this->whereCheckActions)){
                    return null;
                }
        $stepsWhere=function(Query $query)use($actionType){
                    if($actionType&&isset( $this->queryWhereOr[$actionType->value()])){
                        $query->whereOr($this->queryWhereOr[$actionType->value()]);
                    }

            foreach ($this->items as $v){
                $query->whereOr(function (Query $query)use($v){
                    $v->authWhere($query);
                });
            }
        };

        $stepWhere=$this->getWhere();
        if(!$stepWhere){
            return $stepsWhere;
        }

        return  function(Query $query)use($stepWhere,$stepsWhere){
            $query->where($stepWhere)->where($stepsWhere);
        };
    }


    /**
     *  设置要判断权限where的操作
     * @param ActionType[] $whereCheckActions
     * @return $this
     */
    public function setWhereCheckActions(array $whereCheckActions): self
    {
        $this->whereCheckActions = $whereCheckActions;
        return $this;
    }


    /**
     * 获取详情页面显示的字段信息
     * @return StepBase
     * @throws \think\Exception
     */
    public function getInfoShowFields(): StepBase
    {
        $info=$this->getInfo();
        if(!$info){
            throw new \think\Exception('缺少数据信息');
        }

        $curStepInfo=$this->getCurrentStep();
        if(!$curStepInfo){
            throw new \think\Exception('未能获取到数据当前步骤信息');
        }
        $fields=clone $curStepInfo;
        $allFields=[];
        foreach ($this->getInfoPastsStep() as $stepInfo){
            array_push($allFields,...$stepInfo);
        }
        $fields->setFields($allFields);
        return $fields;
    }


    /**
     * 获取数据之前经历过的步骤
     * @return StepBase[]
     * @throws \think\Exception
     */
    public function getInfoPastsStep(){
        $info=$this->getInfo();
        if(!$info){
            throw new \think\Exception('缺少数据信息');
        }
        if($info::hasStepPastsField()&&$info[$info::getStepPastsField()]!==''){
            $stepVals= explode(',',$info[$info::getStepPastsField()]);
        }else{
            if(empty($info[$info::getStepField()])){
                throw new \think\Exception('数据缺少步骤字段');
            }
            $stepVals=getStepPasts($info[$info::getStepField()]);
            if(empty($stepVals)){
                throw new \think\Exception('数据未能获取到经历的步骤信息');
            }
        }

        $steps=[];
        foreach ($this->items as $stepInfo){
            if(in_array($stepInfo::name(),$stepVals,true)){
                $steps[]=$stepInfo;
            }
        }
        return $steps;
    }

}