<?php

namespace tpScriptVueCurd\traits;

use think\Exception;
use think\facade\Event;
use think\model\Collection;
use tpScriptVueCurd\base\model\BaseModel;

trait CurdParams
{

    private ?BaseModel $info=null;
    private ?Collection $infoList=null;

    public function __clone()
    {
        if($this->info){
            Event::listen('onSetInfoByController',function ($params){
                if(isset($this->info->controller)&&$this->info->controller===$params[0]){
                    $this->setInfo($params[1]);
                }
            });
            Event::listen('onSetInfoListByController',function ($params){
                if(isset($this->info->controller)&&$this->info->controller===$params[0]){
                    $this->setInfoList($params[1]);
                }
            });
        }
    }

    /**
     * 获取数据信息
     * @param string|null $key
     * @return BaseModel|string|int|array|float|null
     */
    public function getInfo(string $key=null): mixed
    {
        if(is_null($this->info)){
            return null;
        }
        return is_null($key)?$this->info:$this->info[$key]??null;
    }

    /**
     * 设置数据信息
     * @param BaseModel|string $info
     * @param mixed|null $value
     * @throws Exception
     */
    public function setInfo(BaseModel|string $info,mixed $value=null): void
    {
        if(is_string($info)){
            if(is_null($this->info)){
                throw new \think\Exception('info为null，无法设置info的属性'.$info);
            }
            if(($this->info[$info]??null)===$value){
                return;
            }
            $this->info[$info]=$value;
        }else if($info===$this->info){
            return;
        }else{
            $this->info = $info;
        }
        $this->onSetInfo($this->info);
    }

    protected function onSetInfo(BaseModel $info): void
    {
    }


    public function getInfoList(): Collection
    {
        return $this->infoList;
    }

    public function setInfoList(Collection $infoList): void
    {
        if($infoList===$this->infoList){
            return;
        }
        $this->infoList = $infoList;
        $this->onSetInfoList($infoList);
    }

    /**
     * @param Collection|BaseModel[]|null $infoList
     * @return void
     */
    public function onSetInfoList(Collection $infoList): void
    {

    }
}