<?php

namespace tpScriptVueCurd\traits\controller;

use tpScriptVueCurd\annotate\AuthTitle;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\enum\ActionType;
use tpScriptVueCurd\enum\ErrorCode;
use tpScriptVueCurd\enum\Ev;
use tpScriptVueCurd\field\EditorField;
use tpScriptVueCurd\field\FilesField;
use tpScriptVueCurd\field\StringField;
use tpScriptVueCurd\FieldCollection;
use tpScriptVueCurd\ModelField;
use tpScriptVueCurd\option\field\edit_on_change\type\Func;
use tpScriptVueCurd\option\FieldDo;
use tpScriptVueCurd\option\step\StepBase;
use tpScriptVueCurd\traits\controller\module\EditModule;

/**
 * @property BaseModel $md
 * @method null|BaseModel getInfo() 不正确提示的问题
 */
trait BaseEdit
{
    use EditModule;

    #[AuthTitle('添加与修改',-90)]
    public function edit(){
        return $this->editFields();
    }


    /**
     * 自动判断下一步，并执行
     * @return mixed
     */
    protected function stepEdit(){
        $this->assign('vueCurdAction','edit');
        return $this->edit();
    }



    protected function editFields(){
        $this->setinfoById($this->request->editId??null);
        $old=$this->getInfo();
        $parentInfo=$old->getParentInfo();

        $formChangeSetField=$this->request->param('formChangeSetField');
        try {
            //不是监听字段的处理这个情况的话，要验证权限
            if(!$formChangeSetField&&!$old->checkRowAuth(empty($old->id) ?ActionType::Add:ActionType::Edit)){
                throw new \think\Exception('不能'.(empty($old->id) ?'新增':'修改').'当前数据信息',ErrorCode::SaveAuthErrot->value());
            }
            //获取是否是下一步
            $isNext=$this->autoGetSaveStepIsNext();
        }catch (\Exception $exception){
            $this->errorOut($exception);
        }


        $fields=$this->getEditFields($isNext);

        if($formChangeSetField){
            $this->formChangeSetField($fields);
        }




        if($this->request->isAjax()){
            $data=$this->request->post();
            empty($old->id)||$data['id']=$old->id;
            if(empty($data['id'])&&$this->getChildControllers()){
                $data[$this->md::parentField()]=$old->getParentInfo()->id??0;
            }

            $this->setPostDataBefore($data);


            $this->md->startTrans();
            try{
                $this->doEditSave($data,$fields);
            }catch (\Exception $e){
                $this->md->rollback();
                $this->errorOut($e);
            }
            $this->md->commit();
            //提交后
            $msg=(empty($old->id)?'添加':($isNext?'提交':'修改')).'成功';
            $this->editCommitAfter($msg);

            $refreshList = $this->request->refreshList ?? ($fields instanceof StepBase ? $fields->step->okRefreshList : false);

            $this->successOut($msg,[
                'data'=>$data,
                'info'=>$this->getInfo(),
                'baseInfo'=>$this->getInfo()->getParentInfo(),
                'refreshList'=> $this->treePidField?true:$refreshList,
            ]);
        }
        $this->editBefore($fields);


        try{
            $this->createEditFetchDataBefore($fields,$isNext);
        }catch (\Exception $e){
            $this->errorOut($e);
        }
        $fetchData=$this->createEditFetchData($fields,$isNext);

        if($parentInfo){
            $fetchData['baseId']=$parentInfo->id;
            $fetchData['baseInfo']=$parentInfo;
            $fetchData['parentField']=$this->md::parentField();
            $fetchData['vueCurdAction']='childEdit';
        }

        $fetchData=$this->beforeEditShow($fetchData);
        return $this->showTpl('edit',$fetchData);
    }

    /**
     * 获取编辑界面显示需要的参数
     * @param FieldCollection|StepBase $fields
     * @param bool $isNext
     * @return array
     */
    protected function createEditFetchData( FieldCollection|StepBase $fields,bool$isNext): array
    {
        $fields->each(function (ModelField $v){$v->fieldOnEditShow();});
        $info=$this->getInfo();

        try{
            FieldDo::doFuncByFields(Ev::EditShow,$fields,$isNext);
            $fields->onEditShow();
        }catch(\Exception $e){
             $this->errorOut($e);
        }


        //$info可能在上面改变为有值
        $data=$info->toArray();
        //只处理地区
        $fields->filter(fn(ModelField $v)=> $v instanceof  FilesField||$v instanceof  EditorField||($v instanceof  StringField&&$v->disengageSensitivity()))->doShowData($data);
        //原信息
        $data['sourceData']=$info->toArray();





        $fieldArr=array_values($fields->rendGroup()->fieldToArrayPageType(ActionType::Edit)->fieldsToArray());
        $groupFields=$fields->groupItems?FieldCollection::groupListByItems($fieldArr):null;



        $fields->each(function (ModelField $field){
            $func=$field->getEditGridBy();
            $func&&$field->grid($func());
        });
        $groupGrids=[];
        foreach ($groupFields?:[''=>$fields->all()] as $k=>$v){
            $func=$fields->getEditGridBy();
            $groupGrids[$k]=$func?$func($v,$k):null;
        }

        return [
            'title'=>$this->title,
            'fields'=>$fieldArr,
            'groupFields'=>$groupFields,
            'groupGrids'=>$groupGrids,
            'info'=>$data,
            'fieldComponents'=>$fields->getComponents(ActionType::Edit),
            'isStepNext'=>$isNext,
            'stepInfo'=>$this->steps?$fields->toArray():null,
        ];
    }

    /**
     * 添加/编辑 已经Commit 后执行
     * @param string $msg
     */
    public function editCommitAfter(string &$msg): void
    {
        $this->md->startTrans();
        try{
            $this->getInfo()->getSaveFields()->onSaveAfterCommited($msg);
            }catch (\Exception $e){
            $this->md->rollback();
            $msg.='，但：'.$e->getMessage();
        }
        $this->md->commit();
    }


    /**
     * 自动判断是否下一步
     * @return bool|null
     * @throws \think\Exception
     */
    protected function autoGetSaveStepIsNext():bool{
        if(!$this->steps){
            //未启用
            return false;
        }
        $info=$this->getInfo();
        if(empty($info->id)){
            $nextStepInfo=$info->getNextStep();
            if($nextStepInfo&&$this->checkEditUrl($nextStepInfo)){
                return true;
            }
            throw new \think\Exception('没有满足新增的步骤',ErrorCode::SaveAuthErrot->value());
        }
        $curStepInfo=$info->getCurrentStep();
        if($curStepInfo){
            if($this->checkEditUrl($curStepInfo)){
                return false;
            }
        }
        $nextStepInfo=$info->getNextStep();
        if($nextStepInfo&&$this->checkEditUrl($nextStepInfo)){
            return true;
        }
        throw new \think\Exception('没有满足编辑或下一步的步骤',ErrorCode::SaveAuthErrot->value());
    }


    /**
     * 判断当前地址是否能对上
     * @param StepBase $stepInfo
     * @return bool
     */
    protected function checkEditUrl(StepBase $stepInfo):bool{
        $canEditActions=$stepInfo->step->canEditActions;
        if($stepInfo->step->listBtnAction){
            $canEditActions[]=$stepInfo->step->listBtnAction;
        }
        $canEditActions=array_filter($canEditActions);
        $canEditActions=array_unique($canEditActions);
        $canEditActions=array_map(fn($v)=>trim(str_replace(' ','',$v),'/'),$canEditActions);
        if(empty($canEditActions)){
            return false;
        }


        $parseNameArr=fn ($v)=> array_unique([
            parse_name($v,0,true),
            parse_name($v,0,false),
            parse_name($v,1,true),
            parse_name($v,1,false),
            ]);


        $app=app('http')->getName();
        $controllerList=[];
        $controller=$this->request->controller();
        $controllerArr=[];
        foreach (explode('.',$controller) as $k=>$v){
            $controllerArr[$k]=$parseNameArr($v);
        }
        if(count($controllerArr)>1){
            foreach ($controllerArr[0] as $key=>$val){
                foreach ($controllerArr[1] as $key1=>$val1){
                    $controllerList[]=$val.'.'.$val1;
                }
            }
        }else{
            $controllerList=current($controllerArr);
        }
        $controllerList=array_unique($controllerList);

        $curActionArr=[];
        foreach ($parseNameArr($this->request->action()) as $v){
            foreach ($controllerList as $val){
                $curActionArr[]=$val.'/'.$v;
            }
        }

        if(!array_intersect($canEditActions,$curActionArr)){
            return false;
        }

        if(empty($stepInfo->step->listBtnActionVar)){
            return  true;
        }
        foreach ($stepInfo->step->listBtnActionVar as $k=>$v){
            if((string)$v!==$this->request->param($k.'/s')){
                return false;
            }
        }
        return true;
    }


    /**
     * 数据库保存操作
     * @param BaseModel $model
     * @param FieldCollection $fields
     * @param BaseModel $old
     * @param BaseModel|null $parentInfo
     * @param array|null $data
     * @param bool $isNext
     * @param array $returnSaveData
     * @return BaseModel|\tpScriptVueCurd\base\model\VueCurlModel
     * @throws \think\Exception
     */
    protected function doEditSave(?array $data,FieldCollection $fields)
    {
        $this->editBefore($fields,$data);
        $fields->each(function (ModelField $v)use(&$data){$v->fieldOnEditSave($data);});

        $old=$this->getInfo();
        $old->setSaveFields($fields);
        $old->setPostData($data);

        $this->setInfo(empty($old->id)?$old->add():$old->edit());
        $this->editAfter();
    }


    /**
     * 编辑字段变更监听后处理（需要PHP处理时）
     * @return void
     */
    protected function formChangeSetField(FieldCollection $fields): void
    {
        try{
            $field=$fields->findByName($this->request->param('formChangeSetField'));
            $editOnChange=$field->editOnChange();
            if(!$editOnChange instanceof Func||!is_callable($editOnChange->func)){
                throw new \think\Exception('字段'.$field->name().'的属性editOnChange设置错误');
            }
            $editOnChangeFunc=$editOnChange->func;
            $editOnChangeReturn=$editOnChangeFunc($this->request->param('form/a',[]));
            $editOnChangeReturn||$editOnChangeReturn=[];
        }catch (\Exception $exception){
            $this->errorOut($exception->getMessage());
        }
        $this->successOut(['form'=>$editOnChangeReturn['form']??null,'fields'=>$editOnChangeReturn['fields']??null]);
    }


    /**
     * 获取编辑页面的相关字段信息
     * @param bool $isNext
     * @return FieldCollection|StepBase
     * @throws \think\Exception
     */
    protected function getEditFields(bool $isNext):FieldCollection|StepBase{
        if(!$this->steps){
            return $this->fields;
        }
        $info=$this->getInfo();
        if(!$isNext){
            $fields=$info->getCurrentStep();
            if(!$fields->checkEditAuth()){
                $this->errorOut('当前数据不能执行此操作-E1',ErrorCode::SaveAuthErrot->value());
            }

        }
        $fields=$info->getNextStep();
        if(!$fields->checkNextAuth()){
            $this->errorOut('当前数据不能执行此操作-E2',ErrorCode::SaveAuthErrot->value());
        }
        return $fields;
    }
}