<?php


namespace tpScriptVueCurd\traits\controller;


use think\Collection;
use think\db\Query;
use think\db\Raw;
use think\facade\Cache;
use think\Request;
use tpScriptVueCurd\annotate\AuthTitle;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\enum\ActionType;
use tpScriptVueCurd\enum\Ev;
use tpScriptVueCurd\field\SelectField;
use tpScriptVueCurd\FieldCollection;
use tpScriptVueCurd\ModelField;
use tpScriptVueCurd\option\FieldDo;
use tpScriptVueCurd\option\index_row_btn\Btn;
use tpScriptVueCurd\option\index_row_btn\OpenBtn;
use tpScriptVueCurd\option\parameters\FunControllerIndexData;
use tpScriptVueCurd\option\parameters\FunControllerIndexPage;
use tpScriptVueCurd\option\parameters\FunControllerListChildBtn;
use tpScriptVueCurd\traits\controller\module\IndexModule;

/**
 * 为了方便有时候子控制器也使用
 * Trait BaseIndex
 * @property Request $request
 * @property FieldCollection $fields
 * @property BaseModel $md
 */
trait BaseIndex
{

    use IndexModule;
    /**
     * 列表默认排序
     * @var string|Raw
     */
    public string|Raw $indexDefaultOrder='id DESC';


    #[AuthTitle('数据列表',-100)]
    public function index(){
        try{
            //设置父表数据
            $parentInfo=$this->indexBeforeGetBase();
        }catch (\Exception $e){
            $this->errorOut($e);
        }

        $this->getInfo()->setParentInfo($parentInfo);

        $childTpl=$this->request->param('child_tpl/d',0);


        //设置分页相关
        $this->indexPageOption=new FunControllerIndexPage();
        $this->setIndexPageOption();

        $fields=$this->fields?:$this->md->getFieldConfig();
        $listFields=$fields->listShowItems();

        if($this->request->isAjax()){
            if($this->request->param('get_left_cate/d',0)===1){
                $leftCate=$this->getLeftCate();
                $leftCate===null||$this->successOut($leftCate->toArray());
            }

            $model=$this->indexListModelWhere();
            $model=$model->order($this->getListOrder());
            $option=$this->indexListSelect($model);
            /**
             * @var BaseModel[]|\think\model\Collection $list
             */
            $list=$option->sourceList;
            $this->setInfoList($list);

            $this->setListDataChildBaseInfo($list);

           $this->setListDataStep($list);
            $this->setListDataRowChildBtn($list);
            $this->setListDataRowAuth($list,$parentInfo);



            //字段钩子
            try{
                $listFields->each(function (ModelField $v)use($list,){$v->fieldOnIndexList($list);});

                $oldInfo=$this->getInfo();
                foreach ($list as $row){
                    $this->setInfo($row);
                    FieldDo::doFuncByFields(Ev::IndexRow,$listFields);
                }
                $this->setInfo($oldInfo?:clone $this->md);
                FieldDo::doFuncByFields(Ev::IndexList,$listFields,$list);
            }catch(\Exception $e){
                $this->errorOut($e);
            }


            $indexInfo=$this->getInfo();
            /**
             * 字段自定义按钮
             */
            $list->each(function (BaseModel $v)use($list,$childTpl){
                $parentInfo=$v->getParentInfo();
                $showBtn=new OpenBtn();
                $showBtn->btnTitle='详情';
                $showBtn->modalTitle='查看 '.$this->title.' 相关信息';
                $showBtn->modalUrl=url('show',['base_id'=>$parentInfo&&!empty($parentInfo->id)?$parentInfo->id:0,'id'=>$v->id])->build();
                if($childTpl){
                    $showBtn->modalOffset='lt';
                }
                $v->showBtn=$showBtn;


                $editBtn=new OpenBtn();
                $editBtn->btnTitle='修改';
                $editBtn->modalTitle='修改 '.$this->title.' 相关信息';
                $editBtn->modalUrl=url('edit',['base_id'=>$parentInfo&&!empty($parentInfo->id)?$parentInfo->id:0,'id'=>$v->id])->build();
                if($childTpl){
                    $editBtn->modalOffset='lt';
                }
                $v->editBtn=$editBtn;


                if($this->isTreeIndex()){
                    $childAddBtn=new OpenBtn();
                    $childAddBtn->btnTitle='添加下级';
                    $childAddBtn->btnColor='#597ef7';
                    $childAddBtn->modalTitle='新增 '.$this->title;
                    $childAddBtn->modalUrl=url('edit',['base_id'=>$parentInfo&&!empty($parentInfo->id)?$parentInfo->id:0,'pid'=>$v->id])->build();
                    if($childTpl){
                        $childAddBtn->modalOffset='lt';
                    }
                    $v->childAddBtn=$childAddBtn;
                }

                $this->setInfo($v);
                $otherBtns=[
                    'before'=>[...$this->getListRowBeforeBtns($list),...$v->getListRowBeforeBtns($list)],
                    'after'=>[...$this->getListRowAfterBtns($list),...$v->getListRowAfterBtns($list)],
                ];
                foreach ($otherBtns as $k=>$vo){
                    foreach ($vo as $key=>$val){
                        $otherBtns[$k][$key]=$val->toArray();
                    }
                }
                $v->otherBtns=$otherBtns;
            });
            $this->setInfo($indexInfo);
            //字段显示处理
            $option->data=$list->toArray();
            foreach ($option->data as $k=>$v){
                $listFields->doShowData($option->data[$k]);
            }


            //控制器数据处理钩子
            try{
                $this->indexData($option);
                foreach ($list as $k=>$v){
                    FieldDo::doFuncByFields(Ev::IndexRowAfter,$listFields);
                }
                FieldDo::doFuncByFields(Ev::IndexListAfter,$listFields,$list);
                if($this->isTreeIndex()){
                    $option->data=$this->listToTree($option->data);
                }
            }catch(\Exception $e){
                $this->errorOut($e);
            }

            $this->successOut($option->toArray());
        }




        try{
            $listFields->each(function (ModelField $v)use($parentInfo){$v->fieldOnIndexShow($parentInfo);});
            $this->getInfo()->getParentInfo()===$parentInfo||$this->getInfo()->setParentInfo($parentInfo);
            //要改fields，可以直接在 indexShowBefore 里面$this->fields
            $this->indexShowBefore();
            //字段钩子触发
            FieldDo::doFuncByFields(Ev::IndexShow,$listFields);
        }catch (\Exception $e){
            $this->errorOut($e);
        }


        //是否显示添加按钮
        try{
            $rowAuthAdd=$this->md->checkRowAuth(ActionType::Add);
        }catch (\Exception $e){
            $rowAuthAdd=false;
        }


        $filterFields=clone $fields;
        $paranFilterData=$this->request->param('filter_data','','requestFilterNull');
        $filterData=$paranFilterData?json_decode($paranFilterData,true):null;
        $showFilter=$this->request->param('show_filter/d',1)===1;
        $this->indexFilterBefore($filterFields,$filterData,$showFilter);
        $this->setIndexFilterAddStep($filterFields,$filterData);




        $listColumns=array_values($listFields->fieldToArrayPageType(ActionType::Index)->toArray());
        $showTableTool=$this->request->param('show_table_tool/d',1)===1;

        $baseId=$parentInfo?$parentInfo->id:0;

        $addBtn=new OpenBtn();
        $addBtn->btnType='primary';
        $addBtn->btnTitle='新增';
        $addBtn->modalTitle='新增 '.$this->title;
        $addBtn->modalUrl=url('edit',['base_id'=>$baseId])->build();
        if($childTpl){
            $addBtn->modalOffset='lt';
        }


        $leftCate=$this->getLeftCate();
        if($childTpl&&$leftCate&&$leftCate->show&&$leftCate->paramName==='base_id'){
            $leftCate->show=false;
        }



        $data=[
            'model'=>get_class($this->md),
            'modelName'=>class_basename($this->md),
            'showCreateTime'=> !$listFields->findByName('create_time', false),
            'indexPageOption'=>$this->indexPageOption,
            'listColumns'=>$listColumns,
            'groupGroupColumns'=>$listFields->groupItems? FieldCollection::groupListByItems($listColumns):null,//不管显示是不是一个组，只要groupItems有，列表就分组
            'defaultUrlTpl'=>url('___URL_TPL___',['base_id'=>$baseId])->build(),//防止其他情况使用
            'listUrl'=>$this->request->url(),
            'delUrl'=>url('del')->build(),
            'downExcelTplUrl'=>url('downExcelTpl',['base_id'=>$baseId])->build(),
            'importExcelTplUrl'=>url('importExcelTpl',['base_id'=>$baseId])->build(),
            'exportUrl'=>url('export',['base_id'=>$baseId])->build(),
            'addBtn'=>$addBtn->toArray(),
            'title'=>$this->title,
            'childs'=>[],//会在BaseHaveChildController中更改
            'filterConfig'=>$filterFields->getFilterShowData(),
            'filter_data'=>$filterData?:null,
            'showFilter'=>$showFilter,
            'showTableTool'=>$showTableTool,
            'cWindow'=>$this->request->param('c_window/a'),
            'childTpl'=>$childTpl,
            'showMultipleSelection'=>null,//true or false  是否显示多选框（null，根据del权限判断）
            'auth'=>[
                'add'=>true,
                'edit'=>true,
                'del'=>true,
                'importExcelTpl'=>false,
                'downExcelTpl'=>false,
                'export'=>false,
                'stepAdd'=>$this->getAuthAdd(),
                'rowAuthAdd'=>$rowAuthAdd,
            ],
            'baseInfo'=>$parentInfo,
            'baseId'=>$baseId,
            'fieldComponents'=>$listFields->getComponents(ActionType::Index),
            'filterComponents'=>$filterFields->getFilterComponents(),
            'stepConfig'=>$this->steps?->getConfig(),
            'toolTitleLeftBtns'=>array_map(static fn (Btn $v)=>$v->toArray(),$this->getToolTitleLeftBtns()),
            'toolTitleRightBtns'=>array_map(static fn (Btn $v)=>$v->toArray(),$this->getToolTitleRightBtns()),
            'toolBtnLeftBtns'=>array_map(static fn (Btn $v)=>$v->toArray(),$this->getToolBtnLeftBtns()),
            'toolBtnRightBtns'=>array_map(static fn (Btn $v)=>$v->toArray(),$this->getToolBtnRightBtns()),
            'leftCate'=>$leftCate?$leftCate->toArray():null,
        ];

        if($this->isTreeIndex()){
            //树形结构的列名
            $data['childrenColumnName']=$this->childrenColumnName;
            $data['indentSize']=$this->indentSize;
            $data['expandAllRows']=$this->expandAllRows;
            $data['isTreeIndex']=true;
        }else{
            //展示树形数据时，每层缩进的宽度，以 px 为单位
            $data['childrenColumnName']='';
            $data['indentSize']=15;
            $data['expandAllRows']=false;
            $data['isTreeIndex']=false;
        }



        if($this->getChildControllers()){
            $this->indexFetchDoChild($data);
        }

        try{
            $this->indexFetch($data);
        }catch (\Exception $e){
            $this->errorOut($e);
        }

        return $this->showTpl('index',$data);
    }



    /**
     * BaseIndex 的列表中对$parentInfo赋值的逻辑
     * @throws \think\Exception
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    protected function indexBeforeGetBase(): ?BaseModel
    {
        if($this->getParentController() === null){
            //如果没有父表
            return null;
        }
        //可以在外面赋值
        $baseId=$this->request->get('base_id/d',0);
        $baseId||$baseId=$this->request->param('base_id/d',0);
        if(empty($baseId)){
            return null;
        }
        $parentInfo=(clone $this->getParentController()->md)->find($baseId);
        if(!$parentInfo){
            throw new \think\Exception('未能获取到相关父表信息');
        }
        return $parentInfo;
    }





    /**
     * 设置列表数据步骤信息
     * @param \think\model\Collection|BaseModel[] $list
     * @param null|array|BaseModel $parentInfo
     * @return Collection|\think\model\Collection
     */
    protected function setListDataStep(\think\model\Collection $list):void{
        if(!$this->steps){
            return;
        }
        $indexInfo=$this->getInfo();
        foreach ($list as $v){
            $this->setInfo($v);

            $currentStepInfo=$v->getCurrentStep();
            if($currentStepInfo){
                $currentStepInfo->onListRow();
                $v->currentStepInfo=$currentStepInfo->toArray();
                $v->currentStepInfoCanDo = $currentStepInfo->checkEditAuth();
            }else{
                $v->currentStepInfo=null;
                $v->currentStepInfoCanDo=false;
            }

            $nextStepInfo=$v->getNextStep();
            if($nextStepInfo){
                //下一步护执行这个
//                $nextStepInfo->onListRow();
                $v->nextStepInfo=$nextStepInfo->toArray();
                $v->nextStepInfoCanDo = $nextStepInfo->checkNextAuth();
            }else{
                $v->nextStepInfo=null;
                $v->nextStepInfoCanDo=false;
            }

        }
        $this->setInfo($indexInfo);
    }

    /**
     * 设置数据 子表各按钮配置
     * @param BaseModel[]|\think\model\Collection $list
     * @return Collection|\think\model\Collection
     */
    protected function setListDataRowChildBtn(\think\model\Collection $list):void{
        $indexInfo=$this->getInfo();
        foreach ($list as $info){
            $this->setInfo($info);
            $childBtns=[];
            foreach ($this->getChildControllers() as $childControlle){
                $childControlle->getInfo()->setParentInfo($info);
                $btn=new FunControllerListChildBtn();
                $childControlle->parentIndexBtn($btn,$info);
                if(!$btn->url){
                    $btn->url=url(
                        str_replace(['\\','._'],['.','.'],
                            parse_name(ltrim(str_replace($this->app->getNamespace().'\\controller\\','',get_class($childControlle)),'\\'))).'/index',
                        ['base_id'=>$info->id,'child_tpl'=>1,'show_filter'=>0,'c_window'=>['f'=>'auto','w'=>'50vw','h'=>'72vh']]
                    )->build();
                }
                //可能有性能问题，所以这个功能默认关闭
                if($childControlle->parentIndexShowAddAuth){
                    try{
                        $rowAuthAdd=$childControlle->md->checkRowAuth(ActionType::Add);
                    }catch (\Exception $e){
                        $rowAuthAdd=false;
                    }
                    $btn->canAdd=$rowAuthAdd&&$childControlle->getAuthAdd(clone $childControlle->md,$info);
                }
                $childBtns[class_basename($childControlle->md)]=$btn;
            }
            $info->childBtns=$childBtns;
        }
        $this->setInfo($indexInfo);
    }

    /**
     * 设置数据 __auth 的值，是否具有增删改的权限
     * @param BaseModel[]|\think\model\Collection $list
     * @param null|array|BaseModel $parentInfo
     * @return Collection|\think\model\Collection
     */
    protected function setListDataRowAuth(\think\model\Collection $list):void{
         $list->map(fn(BaseModel $info)=>$info->rowSetAuth([ActionType::Show,ActionType::Edit,ActionType::Del]));
    }

    /**
     * 如果启用了步骤，在列表筛选中会出现当前步骤筛选
     * @param FieldCollection $filterFields
     * @param array|null $filterData
     * @throws \think\Exception
     */
    protected function setIndexFilterAddStep(FieldCollection $filterFields,?array $filterData): void
    {
        if(!$this->steps){
            return;
        }
        if(!$this->steps->getConfig('showFilter')){
            return;
        }

        $steps=[];
        foreach ($this->steps as $v){
            $steps[$v::name()]=$v::title();
        }


        $nextStepFieldName=$this->md::getNextStepField();
        if($nextStepFieldName&&in_array($nextStepFieldName,$this->md::getTableFields())){
            $nextStepField=SelectField::init($nextStepFieldName,'下一个步骤')->multiple(true)->items($steps);
            if($this->steps->getConfig('nextFilterDefaultShow')){
                $nextStepField->filterShow();
            }else{
                $nextStepField->filterHide();
            }
            $nextStepField->filter()->multiple(true);
            $nextStepField->on(Ev::IndexFilterBefore,function (ModelField $field, Query $query, array &$filterData){
                $nextStepFieldName=$field->name();
                if(empty($filterData[$nextStepFieldName])){
                    return;
                }
                $val=$filterData[$nextStepFieldName];
                unset($filterData[$nextStepFieldName]);
                is_array($val)||$val=[$val];
                $query->whereIn($nextStepFieldName,$val);
            });

            $filterFields->unshift($nextStepField);
        }



        if($this->md::hasCurrentStepField()){
            $stepNameIsCurrentStep=true;
            $stepFieldName=$this->md::getCurrentStepField();
        }else{
            $stepNameIsCurrentStep=false;
            $stepFieldName=$this->md::getStepField();
        }


        $stepField=SelectField::init($stepFieldName,'当前步骤')->multiple(true)->items($steps);
        if($stepConfig['currentFilterDefaultShow']){
            $stepField->filterShow();
        }else{
            $stepField->filterHide();
        }
        $stepField->filter()->multiple(true);
        $stepField->on(Ev::IndexFilterBefore,function (ModelField $field, Query $query, array &$filterData)use($stepNameIsCurrentStep){
            $stepFieldName=$field->name();
            if(empty($filterData[$stepFieldName])){
                return;
            }
            $val=$filterData[$stepFieldName];
            unset($filterData[$stepFieldName]);

            is_array($val)||$val=[$val];

            if($stepNameIsCurrentStep){
                $query->whereIn($stepFieldName,$val);
            }else{
                $sqls=[];
                foreach ($val as $stepVal){
                    $stepVal=str_replace(["'",'\\','"'],'',$stepVal);
                    $sqls[]="JSON_EXTRACT(`$stepFieldName`,CONCAT(\"$[\",JSON_LENGTH(`$stepFieldName` ->> '$')-1,\"].step\"))='$stepVal'";
                }
                $query->whereRaw(implode(" OR ",$sqls));
            }
        });

        $filterFields->unshift($stepField);
    }



    /**
     * 步骤--是否能添加
     * @param BaseModel|null $info
     * @param BaseModel|null $parentInfo
     * @return bool
     * @throws \think\Exception
     */
    protected function getAuthAdd():bool{
        if(!$this->steps){
            return true;
        }
        $info=$this->getInfo();
        $nextStep=$info->getNextStep();
        if(!$nextStep||$nextStep->getBeforeStepClassByInfo()!==''){
            return false;
        }
        return $nextStep->checkNextAuth();
    }




    /**
     * index的列表数据查询
     * @param Query|BaseModel $model
     * @return FunControllerIndexData
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    protected function indexListSelect(Query|BaseModel$model):FunControllerIndexData{
        $indexGuid=$this->request->param('pageGuid');
        if($indexGuid){
            //记录导出url
            Cache::tag('indexExportSql')->set('indexExportSql-'.$indexGuid,(clone $model)->fetchSql(true)->select(),60*60*24);
        }

        $option=new FunControllerIndexData();
        $option->model=clone $model;
        if($this->indexPageOption->pageSize>0&&!$this->isTreeIndex()){
            $pageSize=$this->indexPageOption->canGetRequestOption?$this->request->param('pageSize/d',$this->indexPageOption->pageSize):$this->indexPageOption->pageSize;
            $list=$model->paginate($pageSize);
            $option->currentPage=$list->currentPage();
            $option->lastPage=$list->lastPage();
            $option->perPage=$list->listRows();
            $option->total=$list->total();
            $option->sourceList=$list->getCollection();
        }else{
            $option->sourceList=$model->select();
            $option->total=$option->sourceList->count();
            $option->currentPage=1;
            $option->lastPage=1;
            $option->perPage=$option->total;
        }
        return $option;
    }


    /**
     * 获取列表排序   Raw 是为了方便重写
     * @return string|Raw
     */
    protected function getListOrder(){
        $sortField=$this->request->param('sortField');
        $sortOrder=$this->request->param('sortOrder','');
        if($sortField&&$sortOrder){
            switch (strtolower($sortOrder)){
                case 'desc':
                case 'asc':
                    break;
                case 'ascend':
                    $sortOrder='ASC';
                    break;
                case 'descend':
                    $sortOrder='DESC';
                    break;
                default:
                    $sortOrder='DESC';
            }
            $order=$sortField.' '.$sortOrder;
        }else{
            $order=$this->indexDefaultOrder;
        }
        return $order;
    }







    /**
     * 列表显示的where条件处理
     * @param BaseModel $model
     * @param BaseModel|null $parentInfo
     * @param string $searchIdKey 是否可根据ID搜索
     * @return Query|BaseModel
     * @throws \think\Exception
     */
    public function indexListModelWhere(string $searchIdKey='id')
    {
        $fields=$this->fields?:$this->md->getFieldConfig();
        $filterFields=clone $fields;
        $filterData=$filterFields->getParamFilterData();
        $showFilter=true;
        $this->indexFilterBefore($filterFields,$filterData,$showFilter);
        $this->setIndexFilterAddStep($filterFields,$filterData);

        return (clone $this->md)
            ->where(function (Query $query){
                $leftCate=$this->getLeftCate();
                if(!$leftCate||!$leftCate->show||$leftCate->paramName==='base_id'){
                    return;
                }
                $query->where($leftCate->where);
            })
            ->where(function (Query $query)use($searchIdKey){
                $parentInfo=$this->getInfo()->getParentInfo();
                $parentInfo === null || $query->where($this->md::parentField(),$parentInfo->id);
                if(!$searchIdKey){
                    return;
                }
                $idStr=$this->request->param($searchIdKey);
                if(empty($idStr)){
                    return;
                }
                $idArr=is_array($idStr)?$idStr:explode(',',$idStr);
                $idArr=array_filter($idArr);
                if(empty($idArr)){
                    return;
                }
                $query->whereIn('id',$idArr);
            })
            ->where(function(Query $query){
                //这里不应该抛出异常
                $this->indexListWhere($query);
            })
            ->where($filterFields->getFilterWhere($filterData))
            ->where(function(Query $query){
                $childFilterData=$this->request->param('childFilterData',null,'requestFilterNull');
                if($childFilterData&&is_string($childFilterData)){
                    $childFilterData=json_decode($childFilterData,true);
                }
                if(empty($childFilterData)){
                    return [];
                }
                if($this->getChildControllers()){
                    foreach ($this->getChildModelObjs() as $childModel){
                        /**
                         * @var BaseModel $childModel
                         */
                        $type=class_basename($childModel);
                        if(!empty($childFilterData[$type])){
                            $sql=$childModel->where($childModel->fields()->getFilterWhere($childFilterData[$type]))->field($childModel::parentField())->buildSql();
                            if($sql!==$childModel->field($childModel::parentField())->buildSql()){
                                $query->whereRaw('id IN '.$sql);
                            }
                        }
                    }
                }


            })
            ->where($this->steps?->doWhere(ActionType::Index));
    }


    /**
     * 列表所有数据的父表设置
     * @param BaseModel[]|\think\model\Collection $list
     * @return void
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    protected function setListDataChildBaseInfo(\think\model\Collection $list):void{
        if($this->getParentController() === null){
            //如果当前没有父表
            return ;
        }
        if($list->isEmpty()){
            //没有子数据，不需要设置父表
            return;
        }

        $parentInfoList=[];
        foreach ($list as $v){
            $parentInfoList[$v[$this->md::parentField()]]=null;
        }

        $parentInfo=$this->getInfo()->getParentInfo();
        if($parentInfo){
            $parentInfoList[$parentInfo['id']]=$parentInfo;
        }
        if(count($parentInfoList)>1||!$parentInfo){
            $baseIds=array_keys(array_filter($parentInfoList));
            if($baseIds){
                foreach ((clone $this->getParentController()->md)->where('id','in',$baseIds)->select() as $val){
                    $parentInfoList[$val->id]=$val;
                }
            }
        }

        foreach ($list as $v){
            $v->setParentInfo($parentInfoList[$val->id]);
        }
    }

}