<?php

namespace tpScriptVueCurd\traits\controller;

use tpScriptVueCurd\annotate\AuthTitle;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\enum\ActionType;
use tpScriptVueCurd\enum\Ev;
use tpScriptVueCurd\FieldCollection;
use tpScriptVueCurd\ModelField;
use tpScriptVueCurd\option\FieldDo;
use tpScriptVueCurd\option\FieldStep;
use tpScriptVueCurd\option\step\StepBase;
use tpScriptVueCurd\traits\controller\module\ShowModule;

/**
 * @property BaseModel $md
 */
trait BaseShow
{
    use ShowModule;

    #[AuthTitle('详细页面',-80)]
    public function show(){
        $this->setinfoById($this->request->showId??null);
        if($this->steps){
            try {
                $fields=$this->steps->getInfoShowFields();
            }catch (\Exception $exception){
                $this->errorOut($exception);
            }
        }else{
            $fields=$this->fields;
        }

        $fields=$fields->filterHideFieldsByShow($this->getInfo())->whenShowSetAttrValByWheres($this->getInfo())->rendGroup();
        try{
            //字段钩子
            FieldDo::doFuncByFields(Ev::ShowInfoBefore,$fields);
        }catch (\Exception $e){
             $this->errorOut($e);
        }




        try{
            $canShow=$this->getInfo()->checkRowAuth(ActionType::Show);
        }catch (\Exception $exception){
             $this->errorOut($exception);
        }
        if($canShow===false){
             $this->errorOut('您不能查看当前数据信息');
        }


        try{
            $fields->each(function (ModelField $v){$v->fieldOnShow();});
            //字段钩子
            FieldDo::doFuncByFields(Ev::ShowInfo,$fields);
        }catch (\Exception $e){
             $this->errorOut($e);
        }


        //控制器钩子
        $this->showBefore($fields);




        return $this->doShow($this->title,$fields);
    }

    /**
     * 方便可以调用其他模型的查看页面（项目开发中可能会用到）
     * @param string $title
     * @param array $info
     * @param FieldCollection $fields
     * @return mixed
     */
    protected function doShow(string $title,FieldCollection $fields){
        $this->assign('thisAction','show');//使用它的js
        $info=$fields->getInfo();
        if(!$info){
            $this->errorOut('缺少数据信息');
        }
        $data=$info->toArray();


        $fields=$fields->filter(fn(ModelField $v)=>$v->showInDetails())->rendGroup();


        $fields->doShowData($data);
        $fields->fieldToArrayPageType(ActionType::Show);
        $fieldArr=array_values($fields instanceof StepBase?$fields->fieldsToArray():$fields->toArray());


        $groupFields=$fields->groupItems?FieldCollection::groupListByItems($fieldArr):null;


        $fields->each(function (ModelField $field){
            $func=$field->getShowGridBy();
            $func&&$field->grid($func($field));
        });
        $groupGrids=[];
        foreach ($groupFields?:[''=>$fields->all()] as $k=>$v){
            $func=$fields->getShowGridBy();
            $groupGrids[$k]=$func?$func($v,$k):null;
        }
        FieldDo::doFuncByFields(Ev::ShowInfoAfter,$fields,$data);

        return $this->showTpl('show',$this->showFetch([
            'title'=>$title,
            'fields'=>$fieldArr,
            'groupFields'=>$groupFields,
            'groupGrids'=>$groupGrids,
            'info'=>$data,
            'fieldComponents'=>$fields->getComponents(ActionType::Show),
        ]));
    }



}