<?php


namespace tpScriptVueCurd\traits\controller;


use think\App;
use think\exception\HttpResponseException;
use think\Request;
use tpScriptVueCurd\field\FilesField;
use tpScriptVueCurd\option\ConfirmException;

/**
 * Trait Vue
 * @property Request $request
 * @property APP $app
 * @package tpScriptVueCurd\traits\controller
 * @author tj 1079798840@qq.com
 */
trait Vue
{
    private static $vueData = [];
    public string $guid;
    protected bool $isVueAction=true;


    /**
     * tp初始化时会执行
     */
    public function initialize()
    {
        parent::initialize();
        if(empty($this->app)){
            $this->app=app();
        }
        if(empty($this->request)){
            $this->request=$this->app->request;
        }
        if(empty($this->guid)){
            $this->guid=create_guid();
        }


        if ($this->isVueAction) {
            $this->app->view->config(['view_suffix' => 'vue']);
            $this->app->view->engine()->layout(static::getTplPath().'layout'.DIRECTORY_SEPARATOR.'default.vue');
        }
        if(empty(self::$vueData['vueCurdAction'])){
            $this->assign('vueCurdAction',$this->request->action());
        }
        $this->assign('vueCurdVersion',vueCurdVersion());
        $this->assign('vueCurdController',$this->request->controller());
        $this->assign('vueCurdModule',$this->app->http->getName());
        $this->assign('guid',$this->guid);
        $this->assign('loginUrl',tpScriptVueCurdGetLoginUrl());
        $this->assign('vueCurdDebug',static::debug());
        $this->assign('themCssPath',tsvcThemCssPath());
    }


    /**
     * 模板变量|JS变量
     * @param string|array $name
     * @param null $value
     */
    public function assign(string|array $name, $value = null): void
    {
        if ($this->isVueAction) {
            if (is_array($name) && is_null($value)) {
                self::$vueData = [...self::$vueData, ...$name];
            } else {
                self::$vueData[$name] = $value;
            }
        }

        $this->app->view->assign($name, $value);
    }


    /**
     * 解析模板
     * @param string $template
     * @param array $vars
     * @return mixed
     */
    public function fetch(string $template = '', array $vars = []): mixed
    {
        if ($this->isVueAction) {
            self::$vueData = [...self::$vueData, ...$vars];
            $this->app->view->assign('vue_data_json', json_encode(self::$vueData));
        }
        return $this->app->view->fetch($template, $vars);
    }


    /**
     * 解析内容
     * @param string $content
     * @param array $vars
     * @return mixed
     */
    public function display(string $content = '', array $vars = []): mixed
    {
        if ($this->isVueAction) {
            self::$vueData = [...self::$vueData, ...$vars];
            $this->app->view->assign('vue_data_json', json_encode(self::$vueData));
        }
        return  $this->app->view->display($content, $vars);
    }


    public function successOut(array|string $dataOrMsg=''):void{
        $this->success(is_array($dataOrMsg)?'':$dataOrMsg,is_array($dataOrMsg)?$dataOrMsg:[]);
    }

    /**
     * 成功
     * @param string $msg
     * @param array $data
     * @param null $url 为了父方法保持一致
     * @param int $wait 为了父方法保持一致
     * @return void
     */
    public function success(string $msg = '', array $data = [], $url = null,int $wait=3): void
    {
        FilesField::setShowFileInfos();
        $result = [
            'code' => 1,
            'msg'  => $msg,
            'data' => $data,
            'url'  => $url,
            'wait'=>$wait,
        ];

        if ( (request()->isJson() || request()->isAjax() || request()->isPost())) {
            throw new HttpResponseException( json($result));
        }
        $tpl=app('config')->get('app.dispatch_success_tmpl');
        if($tpl){
            $this->app->view->engine()->layout(false);
            throw new HttpResponseException( view($tpl, $result));
        }
        throw new HttpResponseException( view(static::getTplPath().'jump/success.vue', ['vue_data_json'=>json_encode($result)]));
    }


    /**
     * 失败且设置 errorCode
     * @param string|\Exception $msg
     * @param int $errorCode
     * @param bool|array $confirm 是否是返回给前台确认框
     * @throws \think\Exception
     */
    public function errorOut(\Exception|string $msg, int $errorCode=0, bool|array $confirm=false):void{
        if($msg instanceof \think\exception\HttpResponseException||$msg instanceof \think\exception\HttpException
            ||is_subclass_of($msg,\think\exception\HttpResponseException::class)
            ||is_subclass_of($msg,\think\exception\HttpException::class)){
            //这种情况直接返回
            throw $msg;
        }


        if(($msg instanceof \Exception)||is_subclass_of($msg,\Exception::class)){
            errorShowThrow($msg);
            $errorCode||$errorCode= $msg->getCode();
            $confirm||$confirm=$msg instanceof ConfirmException?[
                'okText'=>$msg->okText,
                'cancelText'=>$msg->cancelText,
                'title'=>$msg->title,
            ]:false;
            $msg=$msg->getMessage();
        }

        $outConfirm=[
            'show'=>false,
//            'okText'=>'确认执行','cancelText'=>'取消','title'=>'操作确认',
        ];
        if($confirm){
            if(empty($errorCode)){
                throw new \think\Exception('confirm为true时，errorCode不可为0，且是要唯一');
            }
            if(((int)$this->request->header('confirm-error-code'))===$errorCode){
                //用户已确认，不再返回前台信息，继续执行下面代码
                return;
            }
            $outConfirm['show']=true;
            if(is_array($confirm)){
                $outConfirm=[...$outConfirm,...$confirm];
            }
        }

        $this->request->errorCode=$errorCode;
        $this->request->errorConfirm=$outConfirm;

        $this->error($msg);
    }


    /**
     * 为了与父控制器的方法保持一致
     * @param string $msg
     * @param array $data
     * @param string|null $url
     * @param int $wait
     * @return void
     */
    protected function error(string $msg = '', array $data = [], string $url = null,int $wait=3): void
    {
        $result = [
            'code' => 0,
            'msg'  => $msg,
            'data' => $data,
            'confirm'=>$this->request->errorConfirm??['show'=>false],
            'errorCode'=>$this->request->errorCode??0,
            'url'  => $url,
            'wait'=>$wait,
        ];
        if ( (request()->isJson() || request()->isAjax() || request()->isPost())) {
            throw new HttpResponseException( json($result));
        }
        $tpl=app('config')->get('app.dispatch_error_tmpl');
        if($tpl){
            $this->app->view->engine()->layout(false);
            throw new HttpResponseException( view($tpl, $result));
        }
        throw new HttpResponseException( view(static::getTplPath().'jump/error.vue', ['vue_data_json'=>json_encode($result)]));
    }




    final public static function getTplPath():string{
        return getVCurdDir().'tpl'.DIRECTORY_SEPARATOR;
    }


    /**
     * 是否js与css调试，子类重写
     * @return bool
     */
    public static function debug():bool{
        return appIsDebug();
    }
}