<?php


namespace tpScriptVueCurd\traits\model;


use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\enum\ActionType;
use tpScriptVueCurd\FieldCollection;
use tpScriptVueCurd\option\ModelInfoAuth;

trait InfoAuth
{


    /**
     * 子类继承重写
     * @param ModelInfoAuth $authCheck
     * @return void
     */
    protected function authCheck(ModelInfoAuth $authCheck):void{

    }



    /**
     * 为数据设置权限判断结果
     * @param array|ActionType[] $types
     * @return $this
     */
    public function rowSetAuth(array $types=[ActionType::Show,ActionType::Edit,ActionType::Add,ActionType::Del]): self
    {
        $authCheck=new ModelInfoAuth();
        $this->authCheck($authCheck);

        $arr=$this->__auth??[];


        foreach ([ActionType::Show,ActionType::Edit,ActionType::Add,ActionType::Del] as $v) {
            if (!in_array($v, $types, true)) {
                continue;
            }
            $funcName=$v->value();
            if (!isset($authCheck->$funcName)) {
                continue;
            }
            if (isset($arr[$funcName]) && $arr[$funcName] === false) {
                continue;
            }
            if (is_callable($authCheck->$funcName)) {
                $func = $authCheck->$funcName;
                $arr[$funcName] = $func();
            } else if (is_bool($authCheck->$funcName)) {
                $arr[$funcName] = $authCheck->$funcName;
            }
        }


        if(empty($arr)){
            unset($this->__auth);
        }else{
            $this->__auth=$arr;
        }
        return $this;
    }


    /**
     * 对某一条数据验证是否可操作
     * @param ActionType $type
     * @return bool
     */
    public function checkRowAuth(ActionType $type): bool
    {
        $this->rowSetAuth([$type]);
        if(!isset($this->__auth)){
            return true;
        }
        return $this->__auth[$type->value()] ?? true;
    }

}