<?php

namespace tpScriptVueCurd\traits\model;

use think\facade\Event;
use think\model\Collection;
use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\base\model\VueCurlModel;
use tpScriptVueCurd\FieldCollection;
use tpScriptVueCurd\option\step\StepBase;

trait ModelParams
{
    /**
     * 父表对象
     * @var VueCurlModel|null
     */
    private ?BaseModel $parentInfo=null;

    ################列表时#####################
    /**
     * 列表数据
     * @var Collection|null
     */
    private ?Collection $infoList=null;


    ################保存时#####################
    /**
     * 更新前的数据
     * @var BaseModel|null
     */
    private  ?BaseModel $oldInfo=null;
    /**
     * post信息
     * @var array|null
     */
    private ?array $postData=null;
    /**
     * 保存的数据信息
     * @var array|null
     */
    private ?array $saveData=null;
    /**
     * 保存的相关字段
     * @var StepBase|FieldCollection|null
     */
    private   StepBase|FieldCollection|null $saveFields=null;
    /**
     * 是否excel保存
     * @var bool
     */
    private bool$isExcelSave=false;



    /**
     * @return BaseModel|string|int|array|float|null
     */
    public function getParentInfo(string $key=null): mixed
    {
        if(is_null($this->parentInfo)){
            return null;
        }
        return is_null($key)?$this->parentInfo:$this->parentInfo[$key]??null;
    }

    /**
     * @param VueCurlModel|null $parentInfo
     */
    public function setParentInfo(?VueCurlModel $parentInfo): void
    {
        if($this->parentInfo===$parentInfo){
            return;
        }
        $this->parentInfo = $parentInfo;
        $this->onSetParentInfo($parentInfo);
        Event::trigger('onSetParentInfo',$this);
    }

    /**
     * 当$this->parentInfo变更时
     * @param VueCurlModel|null $parentInfo
     * @return void
     */
    protected function onSetParentInfo(?VueCurlModel $parentInfo): void
    {

    }


    /**
     * 获取index列表数据
     * @return Collection|null
     */
    public function getInfoList(): ?Collection
    {
        return $this->infoList;
    }

    /**
     * 设置index列表数据
     * @param Collection|BaseModel[] $infoList
     * @return void
     */
    public function setInfoList(Collection $infoList): void
    {
        if($this->infoList===$infoList){
            return;
        }
        $this->infoList = $infoList;
        foreach ($infoList as $v){
            $v->setInfoList($infoList);
            $v->getInitConfig()->setInfoList($infoList);
        }
        $this->onSetInfoList($infoList);
    }

    /**
     * 当$this->infoList更改时
     * @param Collection|null $infoList
     * @return void
     */
    protected function onSetInfoList(?Collection $infoList): void
    {

    }






    /**************保存时***************/

    /**
     * 获取数据更新前信息
     * @returnBaseModel|string|int|array|float|null
     */
    public function getOldInfo(string $key=null): mixed
    {
        if(is_null($this->oldInfo)){
            return null;
        }
        return is_null($key)?$this->oldInfo:$this->oldInfo[$key]??null;
    }

    /**
     * 设置数据更新前信息
     * @param BaseModel|null $oldInfo
     * @return void
     */
    protected function setOldInfo(?BaseModel $oldInfo): void
    {
        $this->oldInfo = $oldInfo;
    }


    /**
     * 获取post数据
     * @return array|null
     */
    public function getPostData(string $key=null): int|string|array|float|null
    {
        if(is_null($this->postData)){
            return null;
        }
        return is_null($key)?$this->postData:$this->postData[$key]??null;
    }

    /**
     * 设置post数据
     * @param array $postData
     * @return void
     */
    public function setPostData(array|string $postData,string|int|float|null $val=null): array
    {
        if(empty($postData)){
            throw new \think\Exception('setPostData 参数不能为空');
        }

        if(is_array($postData)&&is_null($val)){
            $this->postData = $postData;
        }else if(is_string($postData)&&!is_null($val)){
            is_null($this->postData)&&$this->postData=[];
            $this->postData[$postData]=$val;
        }else{
            throw new \think\Exception('setPostData 参数错误'.$postData.'='.$val);
        }

        return $this->postData;
    }


    /**
     * 获取保存的数据
     * @return array|null
     */
    public function getSaveData(string $key=null): int|string|array|float|null
    {
        if(is_null($this->saveData)){
            return null;
        }
        return is_null($key)?$this->saveData:$this->saveData[$key]??null;
    }

    /**
     * 设置保存的数据
     * @param array|null $saveData
     * @return void
     */
    public function setSaveData(array|string $saveData,string|int|float|null $val=null): array
    {
        if($saveData===''){
            throw new \think\Exception('setSaveData 参数不能为空字符串');
        }

        if(is_array($saveData)&&is_null($val)){
            $this->saveData = $saveData;
        }else if(is_string($saveData)&&!is_null($val)){
            is_null($this->saveData)&&$this->saveData=[];
            $this->saveData[$saveData]=$val;
        }else{
            throw new \think\Exception('setSaveData 参数错误'.$saveData.'='.$val);
        }

        return $this->saveData;
    }

    public function rmSaveDataKey(string $key):array{
        is_null( $this->saveData)&& $this->saveData=[];
        unset( $this->saveData[$key]);
        return  $this->saveData;
    }

    /**
     * 获取保存的字段信息
     * @return FieldCollection|StepBase|null
     */

    public function getSaveFields(): FieldCollection|StepBase|null
    {
        return $this->saveFields;
    }

    /**
     * 设置保存的数据信息
     * @param FieldCollection|StepBase $saveFields
     * @return void
     */
    public function setSaveFields(FieldCollection|StepBase $saveFields): void
    {
        $this->saveFields = $saveFields;
    }


    /**
     * 获取是否excel保存
     * @return bool
     */
    public function getIsExcelSave(): bool
    {
        return $this->isExcelSave;
    }

    /**
     * 设置是否excel保存
     * @param bool $isExcelSave
     * @return void
     */
    public function setIsExcelSave(bool $isExcelSave): void
    {
        $this->isExcelSave = $isExcelSave;
    }
}