<?php


namespace tpScriptVueCurd\traits\model;



use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\enum\ActionType;
use tpScriptVueCurd\enum\Ev;
use tpScriptVueCurd\FieldCollection;
use tpScriptVueCurd\ModelField;
use tpScriptVueCurd\option\FieldDo;
use tpScriptVueCurd\option\FieldStep;
use tpScriptVueCurd\option\step\StepBase;

/**
 * Trait ModelSave
 * @package tpScriptVueCurd\traits\model
 * @author tj 1079798840@qq.com
 */
trait ModelSave
{

    /**
     * 添加基本事项
     * @return $this
     * @throws \think\Exception
     */
    public function add():self{
        #########################################################################################
        ######  此方法不能有数据库查询操作，要获取其他数据，一律传参。因为我批量添加的时候也是执行此方法  ######
        #########################################################################################
        $this->setOldInfo($this);
        if(is_null($this->getPostData())){
            $this->setPostData([]);
        }
        if(!$this->checkRowAuth(ActionType::Add)){
            throw new \think\Exception('不能添加此栏目信息');
        }



        $this->getSaveFields()&&FieldDo::doFuncByFields(Ev::SaveDataCheckBefore,$this->getSaveFields());
        $this->doSaveData();
        $this->getSaveFields()&&FieldDo::doFuncByFields(Ev::SaveBefore,$this->getSaveFields());



        //没有设置当前步骤， excel导入不分步骤
        if($this->getSaveFields() instanceof StepBase){
            //如果启用了步骤
            if(!static::hasStepField()){
                throw new \think\Exception('未设置步骤字段');
            }

            if(is_null($this->getSaveData(static::getStepField()))){
                $this->getSaveFields()->onSaveBefore();//可能在这里设置了 static::getStepField() 的值
                //doSaveBefore中可能更改了步骤
                is_null($this->getSaveData(static::getStepField()))&&$this->setSaveData(static::getStepField(),$this->getSaveFields()->getStepSaveJson());
            }
        }

        if($this->getSaveData(static::getStepField())){
            //为了防止赋值错误，修正为正确的步骤的值，主要是back
            $this->setSaveData(static::getStepField(),StepBase::correctSteps($this->getSaveData(static::getStepField())));
            if(static::hasCurrentStepField()){
                $this->setSaveData(static::getCurrentStepField(),endStepVal($this->getSaveData(static::getStepField())));
            }
            if(static::hasStepPastsField()){
                $pasts=getStepPasts($this->getSaveData(static::getStepField()));
                if(is_null($pasts)){
                    throw new \think\Exception('获取数据执行过步骤错误');
                }
                $this->setSaveData(static::getStepPastsField(),implode(',',$pasts));
            }
        }




        if(!empty($this->getParentInfo()?->id)){
            $this->setSaveData(static::parentField(),$this->getParentInfo()->id);
        }

        if(static::getCreateLoginUserField()){
            $this->setSaveData(static::getCreateLoginUserField(),staticTpScriptVueCurdGetLoginData()['id']);
        }
        if(static::getUpdateLoginUserField()){
            $this->setSaveData(static::getUpdateLoginUserField(),staticTpScriptVueCurdGetLoginData()['id']);
        }
        if(empty($this->getSaveData($this->createTime))){
            $this->setSaveData($this->createTime,time());
        }

        $this->onSaveBefore();

        $info=clone $this;
        $info->replace(false)->save($this->getSaveData());
        if($this->getSaveFields()){
            $this->getSaveFields()->setInfo($info);
            FieldDo::doFuncByFields(Ev::SaveAfter, $this->getSaveFields());
        }
        $this->onSaveAfter();
        if($this->getSaveFields() instanceof StepBase){
            $this->getSaveFields()->onSaveAfter();
            if(static::hasNextStepField()){
                $nestStep=$info->getNextStep();
                $info[static::getNextStepField()]=$nestStep===null?'':$nestStep::name();
            }
        }
        //可能还有更改
        $info->save();
        return $info;
    }

    /**
     * 修改数据
     * @param array $postData 要保存的数据
     * @param FieldCollection $fields
     * @param BaseModel|null $parentInfo
     * @param BaseModel $beforeInfo
     * @param array $returnSaveData
     * @return $this
     * @throws \think\Exception
     */
    public function edit(): self
    {
        $this->setOldInfo($this);
        if(is_null($this->getPostData())){
            $this->setPostData([]);
        }
        if(!empty($this->id)&&!$this->checkRowAuth(ActionType::Edit)){
            throw new \think\Exception('您不能修改当前数据信息');
        }
        if($this->getSaveFields()&&$this->getSaveFields()->count()>0){
            FieldDo::doFuncByFields(Ev::SaveDataCheckBefore,$this->getSaveFields());
        }

        $this->doSaveData();
        if(empty($this->getPostData('id'))){
            throw new \think\Exception('缺少ID');
        }
        $this->setSaveData('id',$this->getPostData('id'));
        $this->getSaveFields()&&FieldDo::doFuncByFields(Ev::SaveBefore,$this->getSaveFields());


        if($this->getSaveFields() instanceof StepBase){
            //如果启用了步骤
            if(!static::hasStepField()){
                throw new \think\Exception('未设置步骤字段');
            }

            if(is_null($this->getSaveData(static::getStepField()))){
                $this->getSaveFields()->onSaveBefore();//可能在这里设置了 static::getStepField() 的值
                //doSaveBefore中可能更改了步骤
                is_null($this->getSaveData(static::getStepField()))&&$this->setSaveData(static::getStepField(),$this->getSaveFields()->getStepSaveJson());
            }
        }

        if($this->getSaveData(static::getStepField())){
            //为了防止赋值错误，修正为正确的步骤的值，主要是back
            $this->setSaveData(static::getStepField(),StepBase::correctSteps($this->getSaveData(static::getStepField())));
            if(static::hasCurrentStepField()){
                $this->setSaveData(static::getCurrentStepField(),endStepVal($this->getSaveData(static::getStepField())));
            }
            if(static::hasStepPastsField()){
                $pasts=getStepPasts($this->getSaveData(static::getStepField()));
                if(is_null($pasts)){
                    throw new \think\Exception('获取数据执行过步骤错误');
                }
                $this->setSaveData(static::getStepPastsField(),implode(',',$pasts));
            }
        }

        foreach (['create_time','update_time','delete_time'] as $v){
            if(!is_null($this->getSaveFields()->getFieldByName($v,false))){
                if(!empty($this->readonly))unset($this->readonly[array_search($v,$this->readonly)]);
            }
        }
        if(static::getCreateLoginUserField()){
            $this->rmSaveDataKey(static::getCreateLoginUserField());
        }
        if(static::getDeleteLoginUserField()){
            $this->rmSaveDataKey(static::getDeleteLoginUserField());
        }
        if(static::getUpdateLoginUserField()){
            $this->setSaveData(static::getUpdateLoginUserField(),staticTpScriptVueCurdGetLoginData()['id']);
        }


        $this->onSaveBefore();
        $info=clone $this;
        $info->save($this->getSaveData());
        if($this->getSaveFields()){
            $this->getSaveFields()->setInfo($info);
            FieldDo::doFuncByFields(Ev::SaveAfter, $this->getSaveFields());
        }
        $this->onSaveAfter();
        if($this->getSaveFields() instanceof StepBase){
            $this->getSaveFields()->onSaveAfter();
            if(static::hasNextStepField()){
                $nestStep=$info->getNextStep();
                $info[static::getNextStepField()]=$nestStep===null?'':$nestStep::name();
            }
        }
        //可能还有更改
        $info->save();
        return $info;
    }


    /**
     * 保存前 验证数据，数据处理
     * @throws \think\Exception
     */
    final protected function doSaveData():void{

        #########################################################################################
        ######  此方法不能有数据库查询操作，要获取其他数据，一律传参。因为我批量添加的时候也是执行此方法  ######
        #########################################################################################
        $this->onSaveDataCheckBefore();
        $this->setSaveData([
            ...$this->getSaveFields()
            ->setSave($this->getPostData())
            ->getSave()
            ,
            ...($this->getSaveData()?:[])
        ]);
    }
}