<?php


namespace tpScriptVueCurd\traits\model;


use tpScriptVueCurd\base\model\BaseModel;
use tpScriptVueCurd\option\FieldStep;
use tpScriptVueCurd\option\step\StepBase;

trait ModelStep
{


    /**
     * false表示未初始化
     * @var StepBase|false|null
     */
    private StepBase|false|null $nextStep=false;
    /**
     * false表示未初始化
     * @var StepBase|false|null
     */
    private  StepBase|false|null $currentStep=false;


    /**
     * 获取下一步
     * @return StepBase|null 是否要获取最新的（重新获取）
     * @throws \think\Exception
     */
    public function getNextStep(bool $getNewStep=false):?StepBase{
        if($this->nextStep===false||$getNewStep){
            $this->nextStep= $this->getStepConfig()?->getNextStep();
        }
        return $this->nextStep;
    }

    /**
     * 获取当前步骤
     * @param bool $getNewStep 是否要获取最新的（重新获取）
     * @return StepBase|null
     */
    public function getCurrentStep(bool $getNewStep=false):?StepBase{
        if($this->currentStep===false||$getNewStep){
            $this->currentStep= $this->getStepConfig()?->getCurrentStep();
        }
        return $this->currentStep;
    }

    
    /**
     * 步奏字段
     * @return string
     */
    public static function getStepField():string{
        return getModelDefaultStepField();
    }

    /**
     * 是否有步骤字段
     * @return bool
     */
    public static function hasStepField():bool{
        $field=static::getStepField();
        return $field&&in_array($field,static::getTableFields(),true);
    }

    /**
     * 下一步步骤字段
     * @return string
     */
    public static function getNextStepField():string{
        return getModelDefaultNextStepField();
    }


    /**
     * 是否有下一步步骤字段
     * @return bool
     */
    public static function hasNextStepField():bool{
        $field=static::getNextStepField();
        return $field&&in_array($field,static::getTableFields(),true);
    }

    /**
     * 当前字段步骤名称
     * @return string
     */
    public static function getCurrentStepField():string{
        return getModelDefaultCurrentStepField();
    }

    /**
     * 是否有当前步骤字段
     * @return bool
     */
    public static function hasCurrentStepField():bool{
        $field=static::getCurrentStepField();
        return $field&&in_array($field,static::getTableFields(),true);
    }

    /**
     * 数据当前流程已走步骤名称集合
     * @return string
     */
    public static function getStepPastsField():string{
        return getModelDefaultStepPastsField();
    }


    /**
     * 是否有当前数据已走流程的字段
     * @return bool
     */
    public static function hasStepPastsField():bool{
        $field=static::getStepPastsField();
        return $field&&in_array($field,static::getTableFields(),true);
    }


}